# pwmodadr.pl - change IP and MAC address in EEPROM file
#
$debug = 0 ;
$netmask = '';
$gateway = '';

$IP_file = '';
$MAC_file = '';

$eeprom_ip_addr = 'eeprom_ip_addr';
$eeprom_ether = 'eeprom_ether';
$eeprom_ip_gateway_addr = 'eeprom_ip_gateway_addr';

$ego = $0;  $ego =~ s/.*[\\\/]//;
$ego =~ s/\.pl$//i;  # remove .pl for Windows

sub usage {
    local($why) = @_;

    print STDERR "usage: $ego project [options] IP_address [MAC_address]\n";
    print STDERR "options:\n";
    print STDERR " -n netmask     - set netmask (e.g., 255.255.255.0)\n";
    print STDERR " -g IP_address  - set default gateway IP address (e.g., 10.1.2.4)\n";
    print STDERR " -Ifile         - update \"file\" with IP_address and netmask\n";
    print STDERR " -Efile         - update \"file\" with MAC_address (if MAC provided)\n";
    print STDERR "where:\n";
    print STDERR " project        - name of PicoWeb project\n";
    print STDERR " IP_address     - set IP address (e.g., 10.1.2.3)\n";
    print STDERR " MAC_address    - set MAC address (e.g., 0.48.162.0.0.1 or 00:30:a2:00:00:01)\n";
    print STDERR "notes:\n";
    print STDERR "(1) MAC_address, netmask and gateway is optional.\n";
    print STDERR "(2) Searches project's link map file.\n";
    print STDERR "(3) Use \"-\" as IP_address to only set MAC address.\n";
    print STDERR "\n$why\n" if ($why ne '');
    exit 1;
}

$n = 0;
while ($ARGV[0] ne '') {
    $opt = $ARGV[0]; shift;
    print "---$n: $opt\n" if ($debug);
    if ($opt =~ /^-/ && $opt ne '-') {
        if ($opt eq '-d') {
            $debug = 1;
        } elsif ($opt eq '-h') {
            &usage('');
        } elsif ($opt =~ /^-I/) {
            $IP_file = substr($opt, 2);
        } elsif ($opt =~ /^-E/) {
            $MAC_file = substr($opt, 2);
        } elsif ($opt =~ /^-G/) {
            $GW_file = substr($opt, 2);
        } elsif ($opt eq '-n') {
            &usage('missing netmask parameter') if ($ARGV[0] eq '');
            $netmask = $ARGV[0];
            shift;
        } elsif ($opt eq '-g') {
            &usage('missing gateway parameter') if ($ARGV[0] eq '');
            $gateway = $ARGV[0];
            shift;
        } else {
            print STDERR "$ego: unknown option '$opt'!\n";
            exit 1;
        }
    } else {
        ++$n;
        if ($n == 1) {
            $project = $opt;
        } elsif ($n == 2) {
            $IP_addr = $opt;
        } elsif ($n == 3) {
            $MAC_addr = $opt;
        } else {
            &usage('Too many parameters!');
        }
    }
}

&usage('missing IP address') if ($IP_addr eq '');
&usage('missing project name') if ($project eq '');

$map_file = "$project.map";
$eeprom_file = "$project.ep";

if ($IP_addr eq '-') {
    $n = @IP = split('[.]', '0.0.0.0');
} else {
    $n = @IP = split('[.]', $IP_addr);
    if ($n != 4) {
        &usage("Bad IP address ($IP_addr)!");
    }
}

if ($netmask ne '') {
    $n = @NM = split('[.]', $netmask);
    if ($n != 4) {
        &usage("Bad netmask ($netmask)!");
    }
}

if ($gateway ne '') {
    $n = @GW = split('[.]', $gateway);
    if ($n != 4) {
        &usage("Bad default gateway IP address ($gateway)!");
    }
}

if ($MAC_addr ne '') {
    $n = @MAC = split('[.]', $MAC_addr);
    if ($n != 6) {
        $n = @MAC = split('[:]', $MAC_addr);
        if ($n != 6) {
            &usage("Bad MAC address ($MAC_addr)!");
        } else {
            for ($i = 0; $i < $n; ++$i) {
                $MAC[$i] = hex($MAC[$i]);
            }
        }
    }
}

print "editing EEPROM file $eeprom_file\n";
if ($netmask ne '') {
        print "  setting IP address/netmask to $IP_addr/$netmask\n";
} else {
    if ($IP_addr ne '-') {
        print "  setting IP address to $IP_addr\n";
    }
}
print "  setting MAC address to $MAC_addr\n" if ($MAC_addr ne '');
print "  setting default gateway IP to $gateway\n" if ($gateway ne '');

$IP_loc = &search_map($map_file, $eeprom_ip_addr);
if ($IP_loc eq '' && $IP_addr ne '-') {
    print "can't find IP address ($eeprom_ip_addr) in map file $map_file!\n";
    exit 1;
}

$NM_loc = $IP_loc + 4;   # netmask follows IP address in EEPROM

$MAC_loc = &search_map($map_file, $eeprom_ether);
if ($MAC_loc eq '') {
    print "can't find MAC address ($eeprom_ether) in map file $map_file!\n";
    exit 1;
}

$GW_loc = &search_map($map_file, $eeprom_ip_gateway_addr);
if ($GW_loc eq '') {
    print "can't find MAC address ($eeprom_ip_gateway_addr) in map file $map_file!\n";
    exit 1;
}

&fixup_ep($eeprom_file);

if ($IP_file ne '' && $IP_addr ne '-') {
    if (open(F, ">$IP_file")) {
        print "  changing IP address and netmask in file \"$IP_file\"\n";
        print F "$IP_addr\t$netmask\t# PicoWeb IP address and netmask\n";
        close(F);
    } else {
        print "can't open IP address file \"$IP_file\"!\n";
        exit 1;
    }
}

if ($MAC_addr && $MAC_file ne '') {
    if (open(F, ">$MAC_file")) {
        print "  changing MAC address in file \"$MAC_file\"\n";
        print F "$MAC_addr\t# PicoWeb MAC (Ethernet) address\n";
        close(F);
    } else {
        print "can't open MAC address file \"$MAC_file\"!\n";
        exit 1;
    }
}

if ($gateway && $GW_file ne '') {
    if (open(F, ">$GW_file")) {
        print "  changing gateway IP address in file \"$GW_file\"\n";
        print F "$gateway\t# PicoWeb default gateway IP address\n";
        close(F);
    } else {
        print "can't open gateway address file \"$GW_file\"!\n";
        exit 1;
    }
}

exit 0;

sub fixup_ep {
    local($ep_file, $pat) = @_;
    local($line, $s);

    $s = '';
    if (!open(EP, "<$ep_file")) {
        print "can't open EEPROM file $ep_file!\n";
        exit 1;
    }
    binmode(EP);
    while (<EP>) {
        $line = $_;
        $line =~ s/[\s\r\n]+$//;
        # 0001:01^M
        if ($line =~ /^([0-9a-f]+):([0-9a-f]+)$/) {
            $loc = hex($1);
            $val = hex($2);
            if ($IP_addr ne '' && $IP_addr ne '-') {
                if (($loc >= $IP_loc) && ($loc < $IP_loc+4)) {
                    $line = sprintf("%04x:%02x", $loc, $IP[$loc-$IP_loc]);
                    print "--->$line\n" if ($debug);
                }
            }
            if ($netmask ne '') {
                if (($loc >= $NM_loc) && ($loc < $NM_loc+4)) {
                    $line = sprintf("%04x:%02x", $loc, $NM[$loc-$NM_loc]);
                    print "--->$line\n" if ($debug);
                }
            } else {
                if (($loc >= $NM_loc) && ($loc < $NM_loc+4)) {
                    $NM[$loc-$NM_loc] = $val;
                    print "---------->$line\n" if ($debug);
                }
            }
            if ($MAC_addr ne '') {
                if (($loc >= $MAC_loc) && ($loc < $MAC_loc+6)) {
                    $line = sprintf("%04x:%02x", $loc, $MAC[$loc-$MAC_loc]);
                    print "--->$line\n" if ($debug);
                }
            }
            if ($gateway ne '') {
                if (($loc >= $GW_loc) && ($loc < $GW_loc+4)) {
                    $line = sprintf("%04x:%02x", $loc, $GW[$loc-$GW_loc]);
                    print "--->$line\n" if ($debug);
                }
            }
        }
        $s .= $line . "\r\n";
    }
    close(EP);
    if ($netmask eq '') {
        $netmask = sprintf("%d.%d.%d.%d", $NM[0], $NM[1], $NM[2], $NM[3]);
        print "-------netmask: $netmask\n" if ($debug);
    }
    if (!open(EP, ">$ep_file")) {
        print "can't write to EEPROM file $ep_file!\n";
        exit 1;
    }
    binmode(EP);
    print EP $s;
    close(EP);
}

sub search_map {
    local($map_file, $pat) = @_;
    local($line);

    if (!open(MAP, "<$map_file")) {
        print "can't open map file $map_file!\n";
        exit 1;
    }
    while (<MAP>) {
        $line = $_;
        $line =~ s/[\s\r\n]+$//;
        ##print "---$line\n";
        if ($line =~ /\s+0x([0-9a-f]+)\s+$pat$/i) {
            $loc = hex($1);
            print "$1 $loc $pat\n" if ($debug);
        }
    }
    close(MAP);
    return $loc;
}
